package com.bocloud.gateway.restful;

import com.bocloud.gateway.domain.GatewayRoute;
import com.bocloud.gateway.route.GatewayRouteHandler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import lombok.RequiredArgsConstructor;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 路由处理器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/routes")
@RequiredArgsConstructor
public class RouteController {

    private final GatewayRouteHandler routeHandler;

    /**
     * 刷新路由信息
     *
     * @return 刷新结果
     */
    @PatchMapping
    Result refresh() {
        this.routeHandler.load();
        return Result.SUCCESS("refresh route success!");
    }

    /**
     * 查询路由信息
     *
     * @return 查询结果
     */
    @GetMapping
    GeneralResult<List<RouteDefinition>> query() {
        List<RouteDefinition> routes = new ArrayList<>(this.routeHandler.getLocalRouteRepository().getRoutes().values());
        return new GeneralResult<>(true, routes, "success");
    }

    /**
     * 添加路由信息
     *
     * @return 添加结果
     */
    @PostMapping
    Result create(GatewayRoute route) {
        this.routeHandler.addRoute(route);
        return Result.SUCCESS("create route success!");
    }

    /**
     * 删除路由信息
     *
     * @param id 路由ID
     * @return 删除结果
     */
    @DeleteMapping("/{id}")
    Result remove(@PathVariable String id) {
        this.routeHandler.deleteRoute(id);
        return Result.SUCCESS("remove route success!");
    }

    /**
     * 更新路由信息
     *
     * @param route 路由
     * @return 更新结果
     */
    @PutMapping
    Result modify(GatewayRoute route) {
        this.routeHandler.updateRoute(route);
        return Result.SUCCESS("modify route success!");
    }
}
