package com.bocloud.gateway.route;

import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionRepository;
import org.springframework.cloud.gateway.support.NotFoundException;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author dmw
 */
@Slf4j
@Component
public class LocalRouteRepository implements RouteDefinitionRepository {

    @Getter
    private final ConcurrentHashMap<String, RouteDefinition> routes;

    @Autowired
    public LocalRouteRepository() {
        this.routes = new ConcurrentHashMap<>();
    }

    @Override
    public Flux<RouteDefinition> getRouteDefinitions() {
        List<RouteDefinition> routeDefinitions = new ArrayList<>();
        routes.forEach((k, v) -> routeDefinitions.add(v));
        return Flux.fromIterable(routeDefinitions);
    }

    @Override
    public Mono<Void> save(Mono<RouteDefinition> route) {
        return route.flatMap(routeDefinition -> {
            routes.put(routeDefinition.getId(), routeDefinition);
            return Mono.empty();
        });
    }

    public Mono<Void> update(Mono<RouteDefinition> route) {
        return route.flatMap(routeDefinition -> {
            routes.remove(routeDefinition.getId());
            routes.put(routeDefinition.getId(), routeDefinition);
            return Mono.empty();
        });
    }

    @Override
    public Mono<Void> delete(Mono<String> routeId) {
        return routeId.flatMap(id -> {
            if (routes.containsKey(id)) {
                routes.remove(id);
                return Mono.empty();
            }
            return Mono.defer(() -> Mono.error(new NotFoundException("路由文件没有找到: " + routeId)));
        });
    }
}
